package dev.fitko.fitconnect.sdk.tools;

import com.nimbusds.jose.jwk.JWK;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class CertTool {

    public static final String OUTPUT_DIR = "outputDir";
    private final Map<String, String> cmdArgs = new HashMap<>();

    final JWKGenerator jwkGenerator;

    public static void main(final String[] args) {

        new CertTool(new JWKGenerator()).run(args);

        /*final Map<String, Object> data = new LinkedHashMap<>();
        data.put("senderConfig", Map.of("clientSecret", "", "clientId", ""));
        data.put("subscriberConfig", Map.of("clientSecret", "", "clientId", "", "privateDecryptionKeyPaths", List.of(), "privateSigningKeyPath", ""));
        data.put("activeEnvironment", DefaultEnvironments.TEST.getEnvironmentName().getName());

        final DumperOptions options = new DumperOptions();
        options.setDefaultFlowStyle(DumperOptions.FlowStyle.BLOCK);
        options.setPrettyFlow(true);

        final Yaml yaml = new Yaml(options);
        final StringWriter writer = new StringWriter();

        yaml.dump(data, writer);
        writer.flush();

        System.out.println(writer);*/
    }

    public CertTool(final JWKGenerator jwkGenerator){
        this.jwkGenerator = jwkGenerator;
    }

    private void run(final String[] args) {

        mapArgs(args);

        final String dir = getDir();

        final JWKPair encryptionKeyPair = jwkGenerator.generateEncryptionKeyPair(4096);
        final JWKPair signatureKeyPair = jwkGenerator.generateSignatureKeyPair(4096);

        writeFile(Path.of(dir, "publicKey_encryption.json"), encryptionKeyPair.getPublicKey());
        writeFile(Path.of(dir, "privateKey_decryption.json"), encryptionKeyPair.getPrivateKey());

        writeFile(Path.of(dir, "publicKey_signature_verification.json"), signatureKeyPair.getPublicKey());
        writeFile(Path.of(dir, "privateKey_signing.json"), signatureKeyPair.getPrivateKey());

    }

    private String getDir() {
        if (cmdArgs.containsKey(OUTPUT_DIR)) {
            return Path.of(cmdArgs.get(OUTPUT_DIR)).toAbsolutePath().toString();
        } else {
            return createTempDir();
        }
    }

    private static String createTempDir() {
        try {
            return Files.createTempDirectory("testJWKs").toFile().getAbsolutePath();
        } catch (final IOException e) {
            System.out.println(e);
            System.exit(0);
        }
        return null;
    }

    private static void writeFile(final Path path, final JWK jwk) {
        try {
            Files.write(path, toBytes(jwk));
            System.out.println("Wrote " + path.getFileName() + " to " + path);
        } catch (final IOException e) {
            System.out.println(e);
            System.exit(0);
        }
    }

    private void mapArgs(final String[] args) {
        if (args.length > 1) {
            System.out.println("Invalid number of arguments \n Use: -o=OutputDirectory");
            System.exit(0);
        } else if (args.length == 1) {
            final String[] s = args[0].split("=");
            if(s.length != 2){
                System.out.println("Invalid option " + args[0] + "\n Use: -o=OutputDirectory");
                System.exit(0);
            }
            if (Objects.equals(s[0], "-o")) {
                cmdArgs.put(OUTPUT_DIR, s[1]);
            } else {
                System.out.println("Invalid option " + s[0] + "\n Use: -o=OutputDirectory");
                System.exit(0);
            }
        }
    }

    private static byte[] toBytes(final JWK jwk) {
        return jwk.toJSONString().getBytes(StandardCharsets.UTF_8);
    }
}
